@@ -79,7 +79,8 @@ def _compare_tables(conn_table_names, metadata_table_names,
             inspector.reflecttable(t, None)
         conn_column_info[(s, tname)] = t
 
-    for s, tname in sorted(existing_tables):
+    for s, tname in sorted(existing_tables, key=lambda x: (x[0] or '', x[1])):
+        s = s or None
         name = '%s.%s' % (s, tname) if s else tname
         metadata_table = tname_to_table[(s, tname)]
         conn_table = existing_metadata.tables[name]
@@ -127,16 +128,10 @@ def _compare_columns(schema, tname, object_filters, conn_table, metadata_table,
             log.info("Detected added column '%s.%s'", name, cname)
 
     for cname in set(conn_col_names).difference(metadata_col_names):
-        rem_col = sa_schema.Column(
-                    cname,
-                    conn_table.c[cname].type,
-                    nullable=conn_table.c[cname].nullable,
-                    server_default=conn_table.c[cname].server_default
-                )
-        if _run_filters(rem_col, cname,
+        if _run_filters(conn_table.c[cname], cname,
                                 "column", True, None, object_filters):
             diffs.append(
-                ("remove_column", schema, tname, rem_col)
+                ("remove_column", schema, tname, conn_table.c[cname])
             )
             log.info("Detected removed column '%s.%s'", name, cname)
 
@@ -460,6 +455,12 @@ def _compare_type(schema, tname, cname, conn_col,
             conn_type, metadata_type, tname, cname
         )
 
+def _render_server_default_for_compare(metadata_default,
+                                        metadata_col, autogen_context):
+    return _render_server_default(
+                    metadata_default, autogen_context,
+                    repr_=metadata_col.type._type_affinity is sqltypes.String)
+
 def _compare_server_default(schema, tname, cname, conn_col, metadata_col,
                                 diffs, autogen_context):
 
@@ -467,8 +468,9 @@ def _compare_server_default(schema, tname, cname, conn_col, metadata_col,
     conn_col_default = conn_col.server_default
     if conn_col_default is None and metadata_default is None:
         return False
-    rendered_metadata_default = _render_server_default(
-                            metadata_default, autogen_context)
+    rendered_metadata_default = _render_server_default_for_compare(
+                    metadata_default, metadata_col, autogen_context)
+
     rendered_conn_default = conn_col.server_default.arg.text \
                             if conn_col.server_default else None
     isdiff = autogen_context['context']._compare_server_default(
@@ -491,6 +493,3 @@ def _compare_server_default(schema, tname, cname, conn_col, metadata_col,
             tname,
             cname
         )
-
-
-
