@@ -1,22 +1,23 @@
-from alembic.script import ScriptDirectory
-from alembic.environment import EnvironmentContext
-from alembic import util, ddl, autogenerate as autogen
 import os
 
+from .script import ScriptDirectory
+from .environment import EnvironmentContext
+from . import util, autogenerate as autogen
+
 def list_templates(config):
     """List available templates"""
 
-    print "Available templates:\n"
+    config.print_stdout("Available templates:\n")
     for tempname in os.listdir(config.get_template_directory()):
-        readme = os.path.join(
+        with open(os.path.join(
                         config.get_template_directory(),
                         tempname,
-                        'README')
-        synopsis = open(readme).next()
-        print "%s - %s" % (tempname, synopsis)
+                        'README')) as readme:
+            synopsis = next(readme)
+        config.print_stdout("%s - %s", tempname, synopsis)
 
-    print "\nTemplates are used via the 'init' command, e.g.:"
-    print "\n  alembic init --template pylons ./scripts"
+    config.print_stdout("\nTemplates are used via the 'init' command, e.g.:")
+    config.print_stdout("\n  alembic init --template pylons ./scripts")
 
 def init(config, directory, template='generic'):
     """Initialize a new scripts directory."""
@@ -64,7 +65,10 @@ def revision(config, message=None, autogenerate=False, sql=False):
     """Create a new revision file."""
 
     script = ScriptDirectory.from_config(config)
-    template_args = {}
+    template_args = {
+        'config': config  # Let templates use config for
+                          # e.g. multiple databases
+    }
     imports = set()
 
     environment = util.asbool(
@@ -73,7 +77,6 @@ def revision(config, message=None, autogenerate=False, sql=False):
 
     if autogenerate:
         environment = True
-        util.requires_07("autogenerate")
         def retrieve_migrations(rev, context):
             if script.get_revision(rev) is not script.get_revision("head"):
                 raise util.CommandError("Target database is not up to date.")
@@ -92,7 +95,8 @@ def revision(config, message=None, autogenerate=False, sql=False):
             template_args=template_args,
         ):
             script.run_env()
-    script.generate_revision(util.rev_id(), message, **template_args)
+    return script.generate_revision(util.rev_id(), message, refresh=True,
+                                    **template_args)
 
 
 def upgrade(config, revision, sql=False, tag=None):
@@ -112,11 +116,11 @@ def upgrade(config, revision, sql=False, tag=None):
     with EnvironmentContext(
         config,
         script,
-        fn = upgrade,
-        as_sql = sql,
-        starting_rev = starting_rev,
-        destination_rev = revision,
-        tag = tag
+        fn=upgrade,
+        as_sql=sql,
+        starting_rev=starting_rev,
+        destination_rev=revision,
+        tag=tag
     ):
         script.run_env()
 
@@ -129,6 +133,8 @@ def downgrade(config, revision, sql=False, tag=None):
         if not sql:
             raise util.CommandError("Range revision not allowed")
         starting_rev, revision = revision.split(':', 2)
+    elif sql:
+        raise util.CommandError("downgrade with --sql requires <fromrev>:<torev>")
 
     def downgrade(rev, context):
         return script._downgrade_revs(revision, rev)
@@ -136,50 +142,93 @@ def downgrade(config, revision, sql=False, tag=None):
     with EnvironmentContext(
         config,
         script,
-        fn = downgrade,
-        as_sql = sql,
-        starting_rev = starting_rev,
-        destination_rev = revision,
-        tag = tag
+        fn=downgrade,
+        as_sql=sql,
+        starting_rev=starting_rev,
+        destination_rev=revision,
+        tag=tag
     ):
         script.run_env()
 
-def history(config):
+def history(config, rev_range=None):
     """List changeset scripts in chronological order."""
 
     script = ScriptDirectory.from_config(config)
-    for sc in script.walk_revisions():
-        if sc.is_head:
-            print
-        print sc
+    if rev_range is not None:
+        if ":" not in rev_range:
+            raise util.CommandError(
+                    "History range requires [start]:[end], "
+                    "[start]:, or :[end]")
+        base, head = rev_range.strip().split(":")
+    else:
+        base = head = None
+
+    def _display_history(config, script, base, head):
+        for sc in script.walk_revisions(
+                                base=base or "base",
+                                head=head or "head"):
+            if sc.is_head:
+                config.print_stdout("")
+            config.print_stdout(sc.log_entry)
+
+    def _display_history_w_current(config, script, base=None, head=None):
+        def _display_current_history(rev, context):
+            if head is None:
+                _display_history(config, script, base, rev)
+            elif base is None:
+                _display_history(config, script, rev, head)
+            return []
+
+        with EnvironmentContext(
+            config,
+            script,
+            fn=_display_current_history
+        ):
+            script.run_env()
+
+    if base == "current":
+        _display_history_w_current(config, script, head=head)
+    elif head == "current":
+        _display_history_w_current(config, script, base=base)
+    else:
+        _display_history(config, script, base, head)
+
 
 def branches(config):
     """Show current un-spliced branch points"""
     script = ScriptDirectory.from_config(config)
     for sc in script.walk_revisions():
         if sc.is_branch_point:
-            print sc
+            config.print_stdout(sc)
             for rev in sc.nextrev:
-                print "%s -> %s" % (
+                config.print_stdout("%s -> %s",
                     " " * len(str(sc.down_revision)),
                     script.get_revision(rev)
                 )
 
-def current(config):
+def current(config, head_only=False):
     """Display the current revision for each database."""
 
     script = ScriptDirectory.from_config(config)
     def display_version(rev, context):
-        print "Current revision for %s: %s" % (
-                            util.obfuscate_url_pw(
-                                context.connection.engine.url),
-                            script.get_revision(rev))
+        rev = script.get_revision(rev)
+
+        if head_only:
+            config.print_stdout("%s%s" % (
+                rev.revision if rev else None,
+                " (head)" if rev and rev.is_head else ""))
+
+        else:
+            config.print_stdout("Current revision for %s: %s",
+                                util.obfuscate_url_pw(
+                                    context.connection.engine.url),
+                                rev)
         return []
 
     with EnvironmentContext(
         config,
         script,
-        fn = display_version
+        fn=display_version
     ):
         script.run_env()
 
@@ -201,11 +250,11 @@ def stamp(config, revision, sql=False, tag=None):
     with EnvironmentContext(
         config,
         script,
-        fn = do_stamp,
-        as_sql = sql,
-        destination_rev = revision,
-        tag = tag
-    ) as env:
+        fn=do_stamp,
+        as_sql=sql,
+        destination_rev=revision,
+        tag=tag
+    ):
         script.run_env()
 
 def splice(config, parent, child):
@@ -215,5 +264,3 @@ def splice(config, parent, child):
 
     """
     raise NotImplementedError()
-
-
