@@ -25,6 +25,7 @@ sqla_07 = _vers > (0, 7, 2)
 sqla_08 = _vers >= (0, 8, 0, 'b2')
 sqla_09 = _vers >= (0, 9, 0)
 sqla_092 = _vers >= (0, 9, 2)
+sqla_094 = _vers >= (0, 9, 4)
 if not sqla_07:
     raise CommandError(
             "SQLAlchemy 0.7.3 or greater is required. ")
@@ -32,6 +33,8 @@ if not sqla_07:
 from sqlalchemy.util import format_argspec_plus, update_wrapper
 from sqlalchemy.util.compat import inspect_getfullargspec
 
+import logging
+log = logging.getLogger(__name__)
 
 try:
     import fcntl
@@ -140,7 +143,13 @@ def write_outstream(stream, *text):
         if not isinstance(t, binary_type):
             t = t.encode(encoding, 'replace')
         t = t.decode(encoding)
-        stream.write(t)
+        try:
+            stream.write(t)
+        except IOError:
+            # suppress "broken pipe" errors.
+            # no known way to handle this on Python 3 however
+            # as the exception is "ignored" (noisily) in TextIOWrapper.
+            break
 
 def coerce_resource_to_filename(fname):
     """Interpret a filename as either a filesystem location or as a package resource.
@@ -165,7 +174,8 @@ def status(_statmsg, fn, *arg, **kw):
         raise
 
 def err(message):
-    msg(message)
+    log.error(message)
+    msg("FAILED: %s" % message)
     sys.exit(-1)
 
 def obfuscate_url_pw(u):
@@ -325,7 +335,15 @@ def _with_legacy_names(translations):
                 metadata)
         decorated = eval(code, {"target": go})
         decorated.__defaults__ = getattr(fn, '__func__', fn).__defaults__
-        return update_wrapper(decorated, fn)
+        update_wrapper(decorated, fn)
+        if hasattr(decorated, '__wrapped__'):
+            # update_wrapper in py3k applies __wrapped__, which causes
+            # inspect.getargspec() to ignore the extra arguments on our
+            # wrapper as of Python 3.4.  We need this for the
+            # "module class proxy" thing though, so just del the __wrapped__
+            # for now. See #175 as well as bugs.python.org/issue17482
+            del decorated.__wrapped__
+        return decorated
 
     return decorate
 
