@@ -11,6 +11,8 @@ from sqlalchemy.types import TIMESTAMP
 from sqlalchemy.dialects import mysql, postgresql
 from sqlalchemy.sql import and_, column, literal_column
 
+from . import patch
+
 from alembic import autogenerate, util, compat
 from . import eq_, eq_ignore_whitespace, requires_092, requires_09, requires_094
 
@@ -260,6 +262,40 @@ class AutogenRenderTest(TestCase):
             ")"
         )
 
+    @patch("alembic.autogenerate.render.MAX_PYTHON_ARGS", 3)
+    def test_render_table_max_cols(self):
+        m = MetaData()
+        t = Table(
+            'test', m,
+            Column('a', Integer),
+            Column('b', Integer),
+            Column('c', Integer),
+            Column('d', Integer),
+        )
+        eq_ignore_whitespace(
+            autogenerate.render._add_table(t, self.autogen_context),
+            "op.create_table('test',"
+            "*[sa.Column('a', sa.Integer(), nullable=True),"
+            "sa.Column('b', sa.Integer(), nullable=True),"
+            "sa.Column('c', sa.Integer(), nullable=True),"
+            "sa.Column('d', sa.Integer(), nullable=True)])"
+        )
+
+        t2 = Table(
+            'test2', m,
+            Column('a', Integer),
+            Column('b', Integer),
+            Column('c', Integer),
+        )
+
+        eq_ignore_whitespace(
+            autogenerate.render._add_table(t2, self.autogen_context),
+            "op.create_table('test2',"
+            "sa.Column('a', sa.Integer(), nullable=True),"
+            "sa.Column('b', sa.Integer(), nullable=True),"
+            "sa.Column('c', sa.Integer(), nullable=True))"
+        )
+
     def test_render_table_w_fk_schema(self):
         m = MetaData()
         t = Table('test', m,
@@ -567,6 +603,20 @@ render:primary_key\n)"""
                     "name='fk1', use_alter=True)"
         )
 
+    def test_render_fk_constraint_w_metadata_schema(self):
+        m = MetaData(schema="foo")
+        t1 = Table('t', m, Column('c', Integer))
+        t2 = Table('t2', m, Column('c_rem', Integer))
+
+        fk = ForeignKeyConstraint([t1.c.c], [t2.c.c_rem], onupdate="CASCADE")
+        if not util.sqla_08:
+            t1.append_constraint(fk)
+
+        eq_ignore_whitespace(
+            re.sub(r"u'", "'", autogenerate.render._render_constraint(fk, self.autogen_context)),
+            "sa.ForeignKeyConstraint(['c'], ['foo.t2.c_rem'], onupdate='CASCADE')"
+        )
+
     def test_render_check_constraint_literal(self):
         eq_ignore_whitespace(
             autogenerate.render._render_check_constraint(
@@ -767,6 +817,46 @@ class RenderNamingConventionTest(TestCase):
             "sa.UniqueConstraint('c', name='q')"
         )
 
+    def test_render_add_index(self):
+        t = Table('test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+        )
+        idx = Index(None, t.c.active, t.c.code)
+        eq_ignore_whitespace(
+            autogenerate.render._add_index(idx, self.autogen_context),
+            "op.create_index(op.f('ix_ct_test_active'), 'test', "
+            "['active', 'code'], unique=False)"
+        )
+
+    def test_render_drop_index(self):
+        t = Table('test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+        )
+        idx = Index(None, t.c.active, t.c.code)
+        eq_ignore_whitespace(
+            autogenerate.render._drop_index(idx, self.autogen_context),
+            "op.drop_index(op.f('ix_ct_test_active'), table_name='test')"
+        )
+
+    def test_render_add_index_schema(self):
+        t = Table('test', self.metadata,
+            Column('id', Integer, primary_key=True),
+            Column('active', Boolean()),
+            Column('code', String(255)),
+            schema='CamelSchema'
+        )
+        idx = Index(None, t.c.active, t.c.code)
+        eq_ignore_whitespace(
+            autogenerate.render._add_index(idx, self.autogen_context),
+            "op.create_index(op.f('ix_ct_CamelSchema_test_active'), 'test', "
+            "['active', 'code'], unique=False, schema='CamelSchema')"
+        )
+
+
     def test_implicit_unique_constraint(self):
         t = Table('t', self.metadata, Column('c', Integer, unique=True))
         uq = [c for c in t.constraints if isinstance(c, UniqueConstraint)][0]
