@@ -1,7 +1,7 @@
 from tests import clear_staging_env, staging_env, eq_, ne_, is_, staging_directory
 from tests import _no_sql_testing_config, env_file_fixture, script_file_fixture, _testing_config
 from alembic import command
-from alembic.script import ScriptDirectory
+from alembic.script import ScriptDirectory, Script
 from alembic.environment import EnvironmentContext
 from alembic import util
 import os
@@ -24,9 +24,11 @@ class GeneralOrderedTests(unittest.TestCase):
         def_ = util.rev_id()
         ne_(abc, def_)
 
-    def test_003_heads(self):
+    def test_003_api_methods_clean(self):
         eq_(env.get_heads(), [])
 
+        eq_(env.get_base(), None)
+
     def test_004_rev(self):
         script = env.generate_revision(abc, "this is a message", refresh=True)
         eq_(script.doc, "this is a message")
@@ -36,6 +38,7 @@ class GeneralOrderedTests(unittest.TestCase):
             os.path.join(env.dir, 'versions', '%s_this_is_a_message.py' % abc), os.F_OK)
         assert callable(script.module.upgrade)
         eq_(env.get_heads(), [abc])
+        eq_(env.get_base(), abc)
 
     def test_005_nextrev(self):
         script = env.generate_revision(def_, "this is the next rev", refresh=True)
@@ -48,6 +51,7 @@ class GeneralOrderedTests(unittest.TestCase):
         assert callable(script.module.upgrade)
         assert callable(script.module.downgrade)
         eq_(env.get_heads(), [def_])
+        eq_(env.get_base(), abc)
 
     def test_006_from_clean_env(self):
         # test the environment so far with a
@@ -60,6 +64,7 @@ class GeneralOrderedTests(unittest.TestCase):
         eq_(abc_rev.revision, abc)
         eq_(def_rev.down_revision, abc)
         eq_(env.get_heads(), [def_])
+        eq_(env.get_base(), abc)
 
     def test_007_no_refresh(self):
         rid = util.rev_id()
@@ -154,6 +159,36 @@ class TemplateArgsTest(unittest.TestCase):
             {"x": "x1", "y": "y2", "z": "z1", "q": "q1"}
         )
 
+    def test_step_ignore_init_py(self):
+        """test that __init__.py is ignored."""
+
+        env_file_fixture("""
+context.configure(dialect_name='sqlite', template_args={"somearg":"somevalue"})
+""")
+        command.revision(self.cfg, message="some rev")
+        script = ScriptDirectory.from_config(self.cfg)
+        path = os.path.join(script.versions, "__init__.py")
+        with open(path, 'w') as f:
+            f.write(
+                "crap, crap -> crap"
+            )
+        command.revision(self.cfg, message="another rev")
+
+        script.get_revision('head')
+
+    def test_is_ignored_filename(self):
+        script = ScriptDirectory.from_config(self.cfg)
+        cases = [
+            '__init__.py',
+            u'__init__.py',
+            u'__init__.pyc',
+            u'__init__.pyx',
+            u'__init__.pyo',
+        ]
+        for case in cases:
+            assert Script._from_filename(
+                script, script.versions, case) is None
+
     def test_tmpl_args_revision(self):
         env_file_fixture("""
 context.configure(dialect_name='sqlite', template_args={"somearg":"somevalue"})
@@ -163,8 +198,10 @@ context.configure(dialect_name='sqlite', template_args={"somearg":"somevalue"})
 revision = ${repr(up_revision)}
 down_revision = ${repr(down_revision)}
 """)
+
         command.revision(self.cfg, message="some rev")
         script = ScriptDirectory.from_config(self.cfg)
+
         rev = script.get_revision('head')
         with open(rev.path) as f:
             text = f.read()
